(*
 * This file holds the translation from "osmesa.h" to Object FPC
 *
 * Translated by Corpsman 02.02.2010
 * no warranty in correctness, or function
 *)

(*
 * Mesa 3-D graphics library
 * Version:  6.5
 *
 * Copyright (C) 1999-2005  Brian Paul   All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * BRIAN PAUL BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
 * AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *)


(*
 * Mesa Off-Screen rendering interface.
 *
 * This is an operating system and window system independent interface to
 * Mesa which allows one to render images into a client-supplied buffer in
 * main memory.  Such images may manipulated or saved in whatever way the
 * client wants.
 *
 * These are the API functions:
 *   OSMesaCreateContext - create a new Off-Screen Mesa rendering context
 *   OSMesaMakeCurrent - bind an OSMesaContext to a client's image buffer
 *                       and make the specified context the current one.
 *   OSMesaDestroyContext - destroy an OSMesaContext
 *   OSMesaGetCurrentContext - return thread's current context ID
 *   OSMesaPixelStore - controls how pixels are stored in image buffer
 *   OSMesaGetIntegerv - return OSMesa state parameters
 *
 *
 * The limits on the width and height of an image buffer are MAX_WIDTH and
 * MAX_HEIGHT as defined in Mesa/src/config.h.  Defaults are 1280 and 1024.
 * You can increase them as needed but beware that many temporary arrays in
 * Mesa are dimensioned by MAX_WIDTH or MAX_HEIGHT.
 *)


Unit uosmesa;

{$MODE objfpc}{$H+}

Interface

Uses
  Classes, SysUtils, gl, dynlibs;

Const
  OSMESA_MAJOR_VERSION = 6;
  OSMESA_MINOR_VERSION = 5;
  OSMESA_PATCH_VERSION = 0;

  (*
   * Values for the format parameter of OSMesaCreateContext()
   * New in version 2.0.
   *)
  OSMESA_COLOR_INDEX = GL_COLOR_INDEX;
  OSMESA_RGBA = GL_RGBA;
  OSMESA_BGRA = $1;
  OSMESA_ARGB = $2;
  OSMESA_RGB = GL_RGB;
  OSMESA_BGR = $4;
  OSMESA_RGB_565 = $5;

  (*
   * OSMesaPixelStore() parameters:
   * New in version 2.0.
   *)
  OSMESA_ROW_LENGTH = $10;
  OSMESA_Y_UP = $11;

  (*
   * Accepted by OSMesaGetIntegerv:
   *)
  OSMESA_WIDTH = $20;
  OSMESA_HEIGHT = $21;
  OSMESA_FORMAT = $22;
  OSMESA_TYPE = $23;
  OSMESA_MAX_WIDTH = $24; (* new in 4.0 *)
  OSMESA_MAX_HEIGHT = $25; (* new in 4.0 *)

Type
  OSMesaContext = Pointer;
  OSMESAproc = pointer;

  (*
   * Create an Off-Screen Mesa rendering context.  The only attribute needed is
   * an RGBA vs Color-Index mode flag.
   *
   * Input:  format - one of OSMESA_COLOR_INDEX, OSMESA_RGBA, OSMESA_BGRA,
   *                  OSMESA_ARGB, OSMESA_RGB, or OSMESA_BGR.
   *         sharelist - specifies another OSMesaContext with which to share
   *                     display lists.  NULL indicates no sharing.
   * Return:  an OSMesaContext or 0 if error
   *)
  TOSMesaCreateContext = Function(format: GLenum; sharelist: OSMesaContext): OSMesaContext; cdecl;

  (*
   * Create an Off-Screen Mesa rendering context and specify desired
   * size of depth buffer, stencil buffer and accumulation buffer.
   * If you specify zero for depthBits, stencilBits, accumBits you
   * can save some memory.
   *
   * New in Mesa 3.5
   *)
  TOSMesaCreateContextExt = Function(format: GLenum; depthBits: GLint; stencilBits: GLint;
    accumBits: GLint; sharelist: OSMesaContext): OSMesaContext; cdecl;
  (*
   * Destroy an Off-Screen Mesa rendering context.
   *
   * Input:  ctx - the context to destroy
   *)
  TOSMesaDestroyContext = Procedure(ctx: OSMesaContext); cdecl;

  (*
   * Bind an OSMesaContext to an image buffer.  The image buffer is just a
   * block of memory which the client provides.  Its size must be at least
   * as large as width*height*sizeof(type).  Its address should be a multiple
   * of 4 if using RGBA mode.
   *
   * Image data is stored in the order of glDrawPixels:  row-major order
   * with the lower-left image pixel stored in the first array position
   * (ie. bottom-to-top).
   *
   * Since the only type initially supported is GL_UNSIGNED_BYTE, if the
   * context is in RGBA mode, each pixel will be stored as a 4-byte RGBA
   * value.  If the context is in color indexed mode, each pixel will be
   * stored as a 1-byte value.
   *
   * If the context's viewport hasn't been initialized yet, it will now be
   * initialized to (0,0,width,height).
   *
   * Input:  ctx - the rendering context
   *         buffer - the image buffer memory
   *         type - data type for pixel components, only GL_UNSIGNED_BYTE
   *                supported now
   *         width, height - size of image buffer in pixels, at least 1
   * Return:  GL_TRUE if success, GL_FALSE if error because of invalid ctx,
   *          invalid buffer address, type!=GL_UNSIGNED_BYTE, width<1, height<1,
   *          width>internal limit or height>internal limit.
   *)
  TOSMesaMakeCurrent = Function(ctx: OSMesaContext; Buffer: Pointer; type_: GLenum; width: GLsizei; height: GLsizei): glboolean; cdecl;

  (*
   * Return the current Off-Screen Mesa rendering context handle.
   *)
  TOSMesaGetCurrentContext = Function(): OSMesaContext; cdecl;

  (*
   * Set pixel store/packing parameters for the current context.
   * This is similar to glPixelStore.
   * Input:  pname - OSMESA_ROW_LENGTH
   *                    specify actual pixels per row in image buffer
   *                    0 = same as image width (default)
   *                 OSMESA_Y_UP
   *                    zero = Y coordinates increase downward
   *                    non-zero = Y coordinates increase upward (default)
   *         value - the value for the parameter pname
   *
   * New in version 2.0.
   *)
  TOSMesaPixelStore = Procedure(pname: GLint; value: GLint); cdecl;

  (*
   * Return an integer value like glGetIntegerv.
   * Input:  pname -
   *                 OSMESA_WIDTH  return current image width
   *                 OSMESA_HEIGHT  return current image height
   *                 OSMESA_FORMAT  return image format
   *                 OSMESA_TYPE  return color component data type
   *                 OSMESA_ROW_LENGTH return row length in pixels
   *                 OSMESA_Y_UP returns 1 or 0 to indicate Y axis direction
   *         value - pointer to integer in which to return result.
   *)
  TOSMesaGetIntegerv = Procedure(pname: glint; Value: PGLint); cdecl;

  (*
   * Return the depth buffer associated with an OSMesa context.
   * Input:  c - the OSMesa context
   * Output:  width, height - size of buffer in pixels
   *          bytesPerValue - bytes per depth value (2 or 4)
   *          buffer - pointer to depth buffer values
   * Return:  GL_TRUE or GL_FALSE to indicate success or failure.
   *
   * New in Mesa 2.4.
   *)
  TOSMesaGetDepthBuffer = Function(c: OSMesaContext; width: PGLint; height: PGLint;
    bytesPerValue: PGLint; buffer: Pointer): GLBoolean; cdecl;

  (*
   * Return the color buffer associated with an OSMesa context.
   * Input:  c - the OSMesa context
   * Output:  width, height - size of buffer in pixels
   *          format - buffer format (OSMESA_FORMAT)
   *          buffer - pointer to depth buffer values
   * Return:  GL_TRUE or GL_FALSE to indicate success or failure.
   *
   * New in Mesa 3.3.
   *)
  TOSMesaGetColorBuffer = Function(c: OSMesaContext; width: PGLint; height: PGLint;
    format: PGLint; buffer: Pointer): GLboolean; cdecl;

  (*
   * Return pointer to the named function.
   * New in Mesa 4.1
   * Return OSMESAproc in 6.3.
   *)
  TOSMesaGetProcAddress = Function(Const funcName: Pchar): OSMESAproc; cdecl;

  (*
   * Enable/disable color clamping, off by default.
   * New in Mesa 6.4.2
   *)
  TOSMesaColorClamp = Procedure(enable: GLboolean); cdecl;

Var
  OSMesaColorClamp: TOSMesaColorClamp = Nil;
  OSMesaGetProcAddress: TOSMesaGetProcAddress = Nil;
  OSMesaGetColorBuffer: TOSMesaGetColorBuffer = Nil;
  OSMesaGetDepthBuffer: TOSMesaGetDepthBuffer = Nil;
  OSMesaPixelStore: TOSMesaPixelStore = Nil;
  OSMesaGetCurrentContext: TOSMesaGetCurrentContext = Nil;
  OSMesaCreateContext: TOSMesaCreateContext = Nil;
  OSMesaCreateContextExt: TOSMesaCreateContextExt = Nil;
  OSMesaDestroyContext: TOSMesaDestroyContext = Nil;
  OSMesaMakeCurrent: TOSMesaMakeCurrent = Nil;
  OSMesaGetIntegerv: TOSMesaGetIntegerv = Nil;

Function InitOSMesa(LibName: String = 'libOSMesa.so.7'): Boolean;

Implementation

Var
  LibHandle: THandle = 0;

Function LoadAndCheck(Name: String; Var success: Boolean): Pointer;
Begin
  result := GetProcAddress(LibHandle, name);
  If result = Nil Then success := false;
End;

Function InitOSMesa(LibName: String): Boolean;
Begin
  result := false;
  If Not Fileexists(Libname) Then
    Raise Exception.create('Error, file does not exists "' + libname + '"');
  If Libhandle <> 0 Then
    FreeLibrary(Libhandle);

  libhandle := LoadLibrary(Libname);
  If libhandle <> 0 Then Begin
    result := True;
    OSMesaGetProcAddress := TOSMesaGetProcAddress(LoadAndCheck('OSMesaGetProcAddress', result));
    OSMesaColorClamp := TOSMesaColorClamp(LoadAndCheck('OSMesaColorClamp', result));
    OSMesaGetColorBuffer := TOSMesaGetColorBuffer(LoadAndCheck('OSMesaGetColorBuffer', result));
    OSMesaGetDepthBuffer := TOSMesaGetDepthBuffer(LoadAndCheck('OSMesaGetDepthBuffer', result));
    OSMesaPixelStore := TOSMesaPixelStore(LoadAndCheck('OSMesaPixelStore', result));
    OSMesaGetCurrentContext := TOSMesaGetCurrentContext(LoadAndCheck('OSMesaGetCurrentContext', result));
    OSMesaCreateContext := TOSMesaCreateContext(LoadAndCheck('OSMesaCreateContext', result));
    OSMesaCreateContextExt := TOSMesaCreateContextExt(LoadAndCheck('OSMesaCreateContextExt', result));
    OSMesaDestroyContext := TOSMesaDestroyContext(LoadAndCheck('OSMesaDestroyContext', result));
    OSMesaMakeCurrent := TOSMesaMakeCurrent(LoadAndCheck('OSMesaMakeCurrent', result));
    OSMesaGetIntegerv := TOSMesaGetIntegerv(LoadAndCheck('OSMesaGetIntegerv', result));
  End;
End;

End.

